@@ -166,6 +166,18 @@ def intermediate_hash(escrow_contract: Contract,
     })
 
 
+def is_trusted_handler(escrow_contract: Contract,
+                       handler_addr: str,
+                       gas_payer: str,
+                       gas: int = GAS_LIMIT) -> bool:
+    return escrow_contract.functions.isTrustedHandler(handler_addr).call({
+        'from':
+        gas_payer,
+        'gas':
+        gas
+    })
+
+
 def launcher(escrow_contract: Contract,
              gas_payer: str,
              gas: int = GAS_LIMIT) -> str:
@@ -438,6 +450,49 @@ class Job:
         handle_transaction(txn_func, *func_args, **txn_info)
         return self.status() == Status.Pending and self.balance() == hmt_amount
 
+    def add_trusted_handlers(self,
+                             handlers: List[str],
+                             gas: int = GAS_LIMIT) -> bool:
+        """Add trusted handlers that can freely transact with the contract and perform aborts and cancels for example.
+
+        >>> credentials = {
+        ... 	"gas_payer": "0x1413862C2B7054CDbfdc181B83962CB0FC11fD92",
+        ... 	"gas_payer_priv": "28e516f1e2f99e96a48a23cea1f94ee5f073403a1c68e818263f0eb898f1c8e5"
+        ... }
+        >>> rep_oracle_pub_key = b"2dbc2c2c86052702e7c219339514b2e8bd4687ba1236c478ad41b43330b08488c12c8c1797aa181f3a4596a1bd8a0c18344ea44d6655f61fa73e56e743f79e0d"
+        >>> job = Job(credentials, manifest)
+        >>> job.launch(rep_oracle_pub_key)
+        True
+
+        Make sure we se set our gas payer as a trusted handler by default.
+        >>> is_trusted_handler(job.job_contract, job.gas_payer, job.gas_payer)
+        True
+
+        >>> trusted_handlers = ['0x61F9F0B31eacB420553da8BCC59DC617279731Ac', '0xD979105297fB0eee83F7433fC09279cb5B94fFC6']
+        >>> job.add_trusted_handlers(trusted_handlers)
+        True
+        >>> is_trusted_handler(job.job_contract, '0x61F9F0B31eacB420553da8BCC59DC617279731Ac', job.gas_payer)
+        True
+        >>> is_trusted_handler(job.job_contract, '0xD979105297fB0eee83F7433fC09279cb5B94fFC6', job.gas_payer)
+        True
+
+        Args:
+            handlers (List[str]): a list of trusted handlers.
+
+        Returns:
+            bool: returns True if trusted handlers have been setup successfully.
+
+        """
+        txn_func = self.job_contract.functions.addTrustedHandlers
+        txn_info = {
+            "gas_payer": self.gas_payer,
+            "gas_payer_priv": self.gas_payer_priv,
+            "gas": gas
+        }
+        func_args = [handlers]
+        handle_transaction(txn_func, *func_args, **txn_info)
+        return True
+
     def bulk_payout(self,
                     payouts: List[Tuple[str, Decimal]],
                     results: Dict,
@@ -517,17 +572,9 @@ class Job:
         ... 	"gas_payer_priv": "28e516f1e2f99e96a48a23cea1f94ee5f073403a1c68e818263f0eb898f1c8e5"
         ... }
         >>> rep_oracle_pub_key = b"2dbc2c2c86052702e7c219339514b2e8bd4687ba1236c478ad41b43330b08488c12c8c1797aa181f3a4596a1bd8a0c18344ea44d6655f61fa73e56e743f79e0d"
-        >>> job = Job(credentials, manifest)
+        
 
-        The escrow contract is in Pending state after setup so it can be aborted.
-        >>> job.launch(rep_oracle_pub_key)
-        True
-        >>> job.setup()
-        True
-        >>> job.abort()
-        True
-
-        The escrow contract is in Partial state after the first payout and it can't be aborted.
+        The escrow contract is in Partial state after a partial bulk payout so it can be aborted.
         >>> job = Job(credentials, manifest)
         >>> job.launch(rep_oracle_pub_key)
         True
@@ -537,12 +584,16 @@ class Job:
         >>> job.bulk_payout(payouts, {}, rep_oracle_pub_key)
         True
         >>> job.abort()
-        False
-        >>> job.status()
-        <Status.Partial: 3>
+        True
 
-        The escrow contract is in Paid state after the second payout and it can't be aborted.
-        >>> payouts = [("0x852023fbb19050B8291a335E5A83Ac9701E7B4E6", Decimal('80.0'))]
+
+        The escrow contract is in Paid state after the a full bulk payout and it can't be aborted.
+        >>> job = Job(credentials, manifest)
+        >>> job.launch(rep_oracle_pub_key)
+        True
+        >>> job.setup()
+        True
+        >>> payouts = [("0x852023fbb19050B8291a335E5A83Ac9701E7B4E6", Decimal('100.0'))]
         >>> job.bulk_payout(payouts, {'results': 0}, rep_oracle_pub_key)
         True
         >>> job.abort()
@@ -550,6 +601,26 @@ class Job:
         >>> job.status()
         <Status.Paid: 4>
 
+
+        Trusted handler should be able to abort an existing contract
+        >>> trusted_handler = "0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809"
+        >>> job = Job(credentials, manifest)
+        >>> job.launch(rep_oracle_pub_key)
+        True
+        >>> job.setup()
+        True
+        >>> job.add_trusted_handlers([trusted_handler])
+        True
+
+        >>> handler_credentials = {
+        ... 	"gas_payer": "0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809",
+        ... 	"gas_payer_priv": "f22d4fc42da79aa5ba839998a0a9f2c2c45f5e55ee7f1504e464d2c71ca199e1",
+        ...     "rep_oracle_priv_key": b"28e516f1e2f99e96a48a23cea1f94ee5f073403a1c68e818263f0eb898f1c8e5"
+        ... }
+        >>> access_job = Job(credentials=handler_credentials, factory_addr=job.factory_contract.address, escrow_addr=job.job_contract.address)
+        >>> access_job.abort()
+        True
+
         Returns:
             bool: returns True if contract has been destroyed successfully.
 
